#ifndef BASE_MEM_H
#define BASE_MEM_H

#include <cstdlib>
#include <memory>

#if defined(__ANDROID__)
#include <malloc.h>
#endif

#include "base/log.h"

namespace base {

inline void AlignedFree(void* mem) {
#if defined(_WIN32)
  _aligned_free(mem);
#else
  free(mem);
#endif
}

namespace internal {

struct ScopedAlignedFree {
  inline void operator()(void* x) const { AlignedFree(x); }
};

}  // namespace internal

template <typename T>
using AlignedMemPtr = std::unique_ptr<T, internal::ScopedAlignedFree>;

template <int kAlignment>
inline void* AlignedAlloc(size_t size) {
  void* ptr = NULL;
#if defined(__ANDROID__)
  ptr = memalign(kAlignment, size);
#elif defined(__linux__)
  if (posix_memalign(&ptr, kAlignment, size))
    ptr = NULL;
#elif defined(_WIN32)
  ptr = _aligned_malloc(size, kAlignment);
#endif
  DCHECK(ptr);
  // DCHECK(((unsigned)ptr & (kAlignment - 1)) == 0);
  return ptr;
}

inline void* AlignedRealloc(void* ptr, size_t old_size, size_t new_size) {
  auto* new_ptr = base::AlignedAlloc<16>(new_size);
  memcpy(new_ptr, ptr, old_size);
  base::AlignedFree(ptr);
  return new_ptr;
}

template <int kAlignment>
inline bool IsAligned(void* ptr) {
  return (reinterpret_cast<uintptr_t>(ptr) & (kAlignment - 1)) == 0U;
}

}  // namespace base

#endif  // BASE_MEM_H
